@@ -1,4 +1,5 @@
 import dataclasses
+import datetime
 import enum
 import functools
 import io
@@ -7,10 +8,11 @@ import typing
 import urllib.parse
 
 import dacite
+import dateutil.parser
 import jsonschema
 import yaml
 
-from . import path_to_json_schema
+from . import default_json_schema_path
 
 dc = dataclasses.dataclass
 
@@ -29,51 +31,79 @@ class SchemaVersion(enum.Enum):
 
 
 class AccessType(enum.Enum):
-    GITHUB = 'github'
-    HTTP = 'http'
-    LOCAL_FILESYSTEM_BLOB = 'localFilesystemBlob'
-    LOCAL_OCI_BLOB = 'localOciBlob'
-    OCI_REGISTRY = 'ociRegistry'
-    RELATIVE_OCI_REFERENCE = 'relativeOciReference'
-    S3 = 's3'
+    GITHUB = 'github' # XXX: new: gitHub/v1
+    LOCAL_BLOB = 'localBlob/v1'
     NONE = 'None'  # the resource is only declared informally (e.g. generic)
+    OCI_BLOB = 'ociBlob/v1'
+    OCI_REGISTRY = 'ociRegistry' # XXX: new: ociArtifact/v1
+    RELATIVE_OCI_REFERENCE = 'relativeOciReference'
+    S3 = 's3' # XXX: new: s3/v1
 
 
-class SourceType(enum.Enum):
-    GIT = 'git'
+# hack: patch enum to accept "aliases"
+# -> the values defined in enum above will be  used for serialisation; the aliases are also
+# accepted for deserialisation
+# note: the `/v1` suffix is _always_ optional (if absent, /v1 is implied)
+AccessType._value2member_map_ |= {
+    'github/v1': AccessType.GITHUB,
+    'localBlob': AccessType.LOCAL_BLOB,
+    'localFilesystemBlob': AccessType.LOCAL_BLOB,
+    'none': AccessType.NONE,
+    'OCIRegistry': AccessType.OCI_REGISTRY,
+    'OCIRegistry/v1': AccessType.OCI_REGISTRY,
+    'ociArtefact': AccessType.OCI_REGISTRY,
+    'ociArtifact': AccessType.OCI_REGISTRY,
+    'ociArtifact/v1': AccessType.OCI_REGISTRY,
+    's3/v1': AccessType.S3,
+}
+
+AccessTypeOrStr = typing.Union[AccessType, str]
 
 
-class ResourceType(enum.Enum):
-    OCI_IMAGE = 'ociImage'
-    COSIGN_SIGNATURE = 'cosignSignature'
-    GENERIC = 'generic'
-
-
-class ResourceRelation(enum.Enum):
-    LOCAL = 'local'
-    EXTERNAL = 'external'
-
-
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
 class ResourceAccess:
-    type: typing.Union[AccessType, str]
+    type: typing.Optional[AccessTypeOrStr] = AccessType.NONE
 
 
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
+class LocalBlobAccess(ResourceAccess):
+    '''
+    a blob that is accessible locally to the component-descriptor
+
+    see: https://github.com/open-component-model/ocm-spec/blob/main/doc/appendix/B/localBlob.md
+    '''
+    type: AccessTypeOrStr = AccessType.LOCAL_BLOB
+    localReference: str
+    mediaType: str = 'application/data'
+    referenceName: typing.Optional[str] = None
+    globalAccess: typing.Optional[str] = None
+
+
+@dc(frozen=True, kw_only=True)
 class OciAccess(ResourceAccess):
+    type: AccessType = AccessType.OCI_REGISTRY
     imageReference: str
 
 
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
+class OciBlobAccess(OciAccess):
+    type: AccessTypeOrStr = AccessType.OCI_BLOB
+    mediaType: str
+    digest: str
+    size: int
+
+@dc(frozen=True, kw_only=True)
 class RelativeOciAccess(ResourceAccess):
     reference: str
+    type: AccessType = AccessType.RELATIVE_OCI_REFERENCE
 
 
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
 class GithubAccess(ResourceAccess):
     repoUrl: str
-    ref: str
+    ref: typing.Optional[str] = None
     commit: typing.Optional[str] = None
+    type: AccessTypeOrStr = AccessType.GITHUB
 
     def __post_init__(self):
         parsed = self._normalise_and_parse_url()
@@ -86,15 +116,6 @@ class GithubAccess(ResourceAccess):
         if not parsed.scheme:
             # prepend dummy-schema to properly parse hostname and path (and rm it again later)
             parsed = urllib.parse.urlparse('dummy://' + self.repoUrl)
-            parsed = urllib.parse.urlunparse((
-                '',
-                parsed.netloc,
-                parsed.path,
-                '',
-                '',
-                '',
-            ))
-            parsed = urllib.parse.urlparse(parsed)
 
         return parsed
 
@@ -108,26 +129,43 @@ class GithubAccess(ResourceAccess):
         return self._normalise_and_parse_url().hostname
 
 
-@dc(frozen=True)
-class HttpAccess(ResourceAccess):
-    url: str
-
-
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
 class S3Access(ResourceAccess):
     bucketName: str
     objectKey: str
+    region: typing.Optional[str] = None
 
 
-@dc(frozen=True)
-class LocalOCIBlobAccess(ResourceAccess):
-    digest: str
+class ArtefactType(enum.Enum):
+    COSIGN_SIGNATURE = 'cosignSignature'
+    GIT = 'git'
+    OCI_IMAGE = 'ociImage'
+    OCI_ARTEFACT = 'ociArtifact/v1'
+    HELM_CHART = 'helmChart/v1'
+    BLOB = 'blob/v1'
 
 
-@dc(frozen=True)
-class LocalFilesystemBlobAccess(ResourceAccess):
-    filename: str
-    mediaType: typing.Optional[str] = None
+# hack: patch enum to accept "aliases"
+# -> the values defined in enum above will be  used for serialisation; the aliases are also
+# accepted for deserialisation
+# note: the `/v1` suffix is _always_ optional (if absent, /v1 is implied)
+ArtefactType._value2member_map_ |= {
+    'blob': ArtefactType.BLOB,
+    'git/v1': ArtefactType.GIT,
+    'ociImage/v1': ArtefactType.OCI_IMAGE,
+    'ociImage': ArtefactType.OCI_IMAGE,
+    'helmChart': ArtefactType.HELM_CHART,
+}
+
+
+# aliases (deprecated!) for backwards-compatibility
+SourceType = ArtefactType
+ResourceType = ArtefactType
+
+
+class ResourceRelation(enum.Enum):
+    LOCAL = 'local'
+    EXTERNAL = 'external'
 
 
 @dc(frozen=True)
@@ -178,6 +216,29 @@ class DigestSpec:
     normalisationAlgorithm: str
     value: str
 
+
+# EXCLUDE_FROM_SIGNATURE used in digest field for normalisationAlgorithm
+# (in combination with NO_DIGEST for hashAlgorithm and value) to indicate
+# the resource content should not be part of the signature
+EXCLUDE_FROM_SIGNATURE = "EXCLUDE-FROM-SIGNATURE"
+
+# NO_DIGEST used in digest field for hashAlgorithm and value
+# (in combination with EXCLUDE_FROM_SIGNATURE for normalisationAlgorithm)
+# to indicate the resource content should not be part of the signature
+NO_DIGEST = "NO-DIGEST"
+
+
+@dc
+class ExcludeFromSignatureDigest(DigestSpec):
+    '''
+    ExcludeFromSignatureDigest is a special digest notation to indicate the resource
+    content should not be part of the signature
+    '''
+    hashAlgorithm: str = NO_DIGEST
+    normalisationAlgorithm: str = EXCLUDE_FROM_SIGNATURE
+    value: str = NO_DIGEST
+
+
 @dc
 class SignatureSpec:
     algorithm: str
@@ -190,14 +251,6 @@ class Signature:
     digest: DigestSpec
     signature: SignatureSpec
 
-class Provider(enum.Enum):
-    '''
-    internal: from repositoryContext-owner
-    external: from 3rd-party (not repositoryContext-owner)
-    '''
-    INTERNAL = 'internal'
-    EXTERNAL = 'external'
-
 
 @dc(frozen=True)
 class Metadata:
@@ -269,7 +322,7 @@ class ComponentIdentity:
     version: str
 
 
-class Artifact:
+class Artifact(LabelMethodsMixin):
     '''
     base class for ComponentReference, Resource, Source
     '''
@@ -344,14 +397,12 @@ class SourceReference(LabelMethodsMixin):
 class Resource(Artifact, LabelMethodsMixin):
     name: str
     version: str
-    type: typing.Union[ResourceType, str]
+    type: typing.Union[ArtefactType, str]
     access: typing.Union[
         # Order of types is important for deserialization. The first matching type will be taken,
         # i.e. keep generic accesses at the bottom of the list
         GithubAccess,
-        HttpAccess,
-        LocalFilesystemBlobAccess,
-        LocalOCIBlobAccess,
+        OciBlobAccess,
         OciAccess,
         RelativeOciAccess,
         S3Access,
@@ -364,21 +415,50 @@ class Resource(Artifact, LabelMethodsMixin):
     labels: typing.List[Label] = dataclasses.field(default_factory=tuple)
     srcRefs: typing.List[SourceReference] = dataclasses.field(default_factory=tuple)
 
-@dc(frozen=True)
+
+@dc(frozen=True, kw_only=True)
 class RepositoryContext:
-    pass  # actually, must have attr `type`
+    type: AccessTypeOrStr
 
 
-class OciComponentNameMapping(enum.Enum):
-    URL_PATH = 'urlPath'
-    SHA256_DIGEST = 'sha256-digest'
-
-
-@dc(frozen=True)
+@dc(frozen=True, kw_only=True)
 class OciRepositoryContext(RepositoryContext):
     baseUrl: str
-    componentNameMapping: OciComponentNameMapping = OciComponentNameMapping.URL_PATH
-    type: AccessType = AccessType.OCI_REGISTRY
+    subPath: typing.Optional[str] = None
+    type: AccessTypeOrStr = AccessType.OCI_REGISTRY
+
+    @property
+    def oci_ref(self):
+        if not self.subPath:
+            return self.baseUrl
+        return f'{self.baseUrl.rstrip("/")}/{self.subPath.lstrip("/")}'
+
+    def component_oci_ref(self, name: typing.Union[str, 'Component', 'ComponentIdentity'], /):
+        if isinstance(name, Component):
+            name = name.name
+        elif isinstance(name, ComponentIdentity):
+            name = name.name
+
+        return '/'.join((
+            self.oci_ref,
+            'component-descriptors',
+            name.lstrip('/').lower(), # oci-spec only allows lowercase
+        ))
+
+    def component_version_oci_ref(
+        self,
+        name: typing.Union[str, 'Component', 'ComponentIdentity'],
+        version: str=None,
+    ):
+        if isinstance(name, Component):
+            name = name.name
+        elif isinstance(name, ComponentIdentity):
+            name = name.name
+
+        if not version:
+            name, version = name.rsplit(':', 1)
+
+        return f'{self.component_oci_ref(name)}:{version}'
 
 
 @dc
@@ -387,7 +467,7 @@ class ComponentSource(Artifact, LabelMethodsMixin):
     access: GithubAccess
     version: typing.Optional[str] = None  # introduce this backwards-compatible for now
     extraIdentity: typing.Dict[str, str] = dataclasses.field(default_factory=dict)
-    type: SourceType = SourceType.GIT
+    type: typing.Union[ArtefactType, str] = ArtefactType.GIT
     labels: typing.List[Label] = dataclasses.field(default_factory=list)
 
 
@@ -397,7 +477,7 @@ class Component(LabelMethodsMixin):
     version: str  # relaxed semver
 
     repositoryContexts: typing.List[OciRepositoryContext]
-    provider: Provider
+    provider: typing.Union[str, dict]
 
     sources: typing.List[ComponentSource]
     componentReferences: typing.List[ComponentReference]
@@ -405,6 +485,8 @@ class Component(LabelMethodsMixin):
 
     labels: typing.List[Label] = dataclasses.field(default_factory=list)
 
+    creationTime: typing.Optional[datetime.datetime] = None
+
     def current_repository_ctx(self):
         if not self.repositoryContexts:
             return None
@@ -442,7 +524,7 @@ class ComponentDescriptor:
         if validation_mode is ValidationMode.NONE:
             return
 
-        json_schema_file_path = json_schema_file_path or path_to_json_schema()
+        json_schema_file_path = json_schema_file_path or default_json_schema_path
         schema_dict = _read_schema_file(json_schema_file_path)
 
         try:
@@ -463,23 +545,35 @@ class ComponentDescriptor:
         component_descriptor_dict: dict,
         validation_mode: ValidationMode = ValidationMode.NONE,
     ):
+        def dateparse(v):
+            if not v:
+                return None
+            if isinstance(v, datetime.datetime):
+                return v
+            return dateutil.parser.isoparse(v)
+
         component_descriptor = dacite.from_dict(
             data_class=ComponentDescriptor,
             data=component_descriptor_dict,
             config=dacite.Config(
                 cast=[
-                    OciComponentNameMapping,
-                    AccessType,
-                    Provider,
-                    ResourceType,
                     SchemaVersion,
-                    SourceType,
                     ResourceRelation,
                 ],
                 type_hooks={
-                    typing.Union[AccessType, str]: functools.partial(enum_or_string, enum_type=AccessType),
-                    typing.Union[ResourceType, str]: functools.partial(enum_or_string, enum_type=ResourceType),
-                    typing.Union[OciComponentNameMapping, str]: functools.partial(enum_or_string, enum_type=OciComponentNameMapping),
+                    typing.Union[AccessType, str]: functools.partial(
+                        enum_or_string, enum_type=AccessType
+                    ),
+                    typing.Union[ArtefactType, str]: functools.partial(
+                        enum_or_string, enum_type=ArtefactType
+                    ),
+                    typing.Union[ArtifactIdentity, str]: functools.partial(
+                        enum_or_string, enum_type=ArtefactType
+                    ),
+                    AccessType: functools.partial(
+                        enum_or_string, enum_type=AccessType
+                    ),
+                    datetime.datetime: dateparse,
                 },
             )
         )
@@ -507,4 +601,6 @@ class EnumValueYamlDumper(yaml.SafeDumper):
     def represent_data(self, data):
         if isinstance(data, enum.Enum):
             return self.represent_data(data.value)
+        if isinstance(data, datetime.datetime):
+            return self.represent_data(data.isoformat(timespec='seconds'))
         return super().represent_data(data)
